# バッチサンプル構築手順
  - バッチ一覧
    1. 利用者作成バッチ : create-user-batch : application000001/2/3
    1. CToken連携バッチ : link-ctoken-batch : application000001/2/3
    1. Region改定再同意依頼バッチ : region-term-of-use-notification-batch : root
    1. Book自動閉鎖バッチ : book-end-of-use-batch : root
    1. Region利用自動終了対象追加 : add-region-end-of-use-target-batch : root
    1. Region利用自動終了バッチ : region-end-of-use-batch : region000001
    1. Book削除バッチ : book-delete-batch : root/region000001/application000001/2/3
    1. 出力データファイル管理作成バッチ : output-user-data-file-mng-batch : root
    1. 利用者データ削除バッチ : delete-user-batch : application000001/2/3
    1. Region終了対象追加 : add-region-close-target-batch : root
    1. Region利用終了 : close-region-batch : root
    1. Region利用者連携 : cooperate-region-batch : region000001

  - バッチのビルドをすること
    - ビルド方法 : 「サンプルバッチ関連ファイル\# バッチサンプルビルド手順.md」を参照すること

  - バッチ一覧に該当するバッチリポジトリとmanifestを手動リリース(push)すること
    - 手動Push
    
## configmapにバッチのcommon-configmapを追加及び設定
    - 手動Push 
    - パス : /root/work/build/manifest/eks/configmap

### 共通

## cronjobの各アクターに対して該当バッチのyaml配置及び設定変更
  - yamlファイル置き場 : 「サンプルバッチ関連ファイル\# sample_batch_manifest.zip
    - ★配置パスは適切な場所に移動後、修正すること
    - S3からのファイル取得 （S3バケットがpxross1-batch-workの場合）: aws s3 cp s3://pxross1-batch-work/sample_batch_manifest.zip ./
    - ZIPファイル解凍コマンド : unzip sample_batch_manifest.zip

### root
　```★の箇所を修正する```
  - region-term-of-use-notification-batch.yaml
```json
apiVersion: batch/v1beta1
kind: CronJob
metadata:
  name: root-region-term-of-use-notification-batch
  namespace: pxr
spec:
  concurrencyPolicy: Forbid
  successfulJobsHistoryLimit: 1
  failedJobsHistoryLimit: 1
  startingDeadlineSeconds: 3600
  schedule: "0 8 * * *"　★必要に応じてスケジュール修正
  jobTemplate:
    spec:
      template:
        spec:
          containers:
          - name: region-term-of-use-notification-batch
            image: ＜ECRのDNS名＞/pxross1:region-term-of-use-notification-batch-container ★ECRのDNS名を修正
            imagePullPolicy: IfNotPresent
            command: ["node", "/usr/src/app/sendRegionTermsOfUseNotification.js"]
            volumeMounts:
            - mountPath: /usr/src/app/efs
              name: data-strage
            - mountPath: /usr/src/app/config
              readOnly: true
              name: region-term-of-use-notification-batch-config
          restartPolicy: OnFailure
          volumes:
          - name: data-strage
            persistentVolumeClaim:
              claimName: efs-cronjoblog-claim
          - name: config-dir
            emptyDir: {}
          - name: common-config
            configMap:
              name: common-batch-config-map
          - name: block-config
            configMap:
              name: root-common-batch-config-map
          - name: region-term-of-use-notification-batch-config
            configMap:
              name: root-region-term-of-use-notification-batch-container-config-map
```

  - book-end-of-use-batch.yaml
```json
apiVersion: batch/v1beta1
kind: CronJob
metadata:
  name: root-book-end-of-use-batch
  namespace: pxr
spec:
  concurrencyPolicy: Forbid
  successfulJobsHistoryLimit: 1
  failedJobsHistoryLimit: 1
  startingDeadlineSeconds: 3600
  schedule: "10 * * * *" ★必要に応じてスケジュール修正
  jobTemplate:
    spec:
      template:
        spec:
          containers:
          - name: book-end-of-use-batch
            image: ＜ECRのDNS名＞/pxross1:book-end-of-use-batch-container ★ECRのDNS名を修正
            imagePullPolicy: IfNotPresent
            command: ["node", "/usr/src/app/bookEndOfUse.js"]
            volumeMounts:
            - mountPath: /usr/src/app/efs
              name: data-strage
            - mountPath: /usr/src/app/config
              readOnly: true
              name: book-end-of-use-batch-config
          restartPolicy: OnFailure
          volumes:
          - name: data-strage
            persistentVolumeClaim:
              claimName: efs-cronjoblog-claim
          - name: config-dir
            emptyDir: {}
          - name: common-config
            configMap:
              name: common-batch-config-map
          - name: block-config
            configMap:
              name: root-common-batch-config-map
          - name: book-end-of-use-batch-config
            configMap:
              name: root-book-end-of-use-batch-container-config-map
```
  - book-end-of-use-batch.yaml
```json
apiVersion: batch/v1beta1
kind: CronJob
metadata:
  name: root-book-delete-batch
  namespace: pxr
spec:
  concurrencyPolicy: Forbid
  successfulJobsHistoryLimit: 1
  failedJobsHistoryLimit: 1
  startingDeadlineSeconds: 3600
  schedule: "0 8 * * *" ★必要に応じてスケジュール修正
  jobTemplate:
    spec:
      template:
        spec:
          containers:
          - name: book-delete-batch
            image: ＜ECRのDNS名＞/pxross1:book-delete-batch-container ★ECRのDNS名を修正
            imagePullPolicy: IfNotPresent
            command: ["node", "/usr/src/app/deleteBook.js"]
            volumeMounts:
            - mountPath: /usr/src/app/efs
              name: data-strage
            - mountPath: /usr/src/app/config
              readOnly: true
              name: book-delete-batch-config
          restartPolicy: OnFailure
          volumes:
          - name: data-strage
            persistentVolumeClaim:
              claimName: efs-cronjoblog-claim
          - name: config-dir
            emptyDir: {}
          - name: common-config
            configMap:
              name: common-batch-config-map
          - name: block-config
            configMap:
              name: root-common-batch-config-map
          - name: book-delete-batch-config
            configMap:
              name: root-book-delete-batch-container-config-map
```

  - add-region-end-of-use-target-batch.yaml
```json
apiVersion: batch/v1beta1
kind: CronJob
metadata:
  name: root-add-region-end-of-use-target-batch
  namespace: pxr
spec:
  concurrencyPolicy: Forbid
  successfulJobsHistoryLimit: 1
  failedJobsHistoryLimit: 1
  startingDeadlineSeconds: 3600
  schedule: "0 * * * *" ★必要に応じてスケジュール修正
  jobTemplate:
    spec:
      template:
        spec:
          containers:
          - name: add-region-end-of-use-target-batch
            image: ＜ECRのDNS名＞/pxross1:add-region-end-of-use-target-batch-container ★ECRのDNS名を修正
            imagePullPolicy: IfNotPresent
            command: ["node", "/usr/src/app/addRegionEndOfUseTarget.js"]
            volumeMounts:
            - mountPath: /usr/src/app/efs
              name: data-strage
            - mountPath: /usr/src/app/config
              readOnly: true
              name: add-region-end-of-use-target-batch-config
          restartPolicy: OnFailure
          volumes:
          - name: data-strage
            persistentVolumeClaim:
              claimName: efs-cronjoblog-claim
          - name: config-dir
            emptyDir: {}
          - name: common-config
            configMap:
              name: common-batch-config-map
          - name: block-config
            configMap:
              name: root-common-batch-config-map
          - name: add-region-end-of-use-target-batch-config
            configMap:
              name: root-add-region-end-of-use-target-batch-container-config-map
```

  - output-user-data-file-mng-batch.yaml
```json
apiVersion: batch/v1beta1
kind: CronJob
metadata:
  name: root-output-user-data-file-mng-batch
  namespace: pxr
spec:
  concurrencyPolicy: Forbid
  successfulJobsHistoryLimit: 1
  failedJobsHistoryLimit: 1
  startingDeadlineSeconds: 3600
  schedule: "15 * * * *" ★必要に応じてスケジュール修正
  jobTemplate:
    spec:
      template:
        spec:
          containers:
          - name: output-user-data-file-mng-batch
            image: ＜ECRのDNS名＞/pxross1:output-user-data-file-mng-batch-container ★ECRのDNS名を修正
            imagePullPolicy: IfNotPresent
            command: ["node", "/usr/src/app/createOutputUserDataFileMng.js"]
            volumeMounts:
            - mountPath: /usr/src/app/efs
              name: data-strage
            - mountPath: /usr/src/app/config
              readOnly: true
              name: output-user-data-file-mng-batch-config
          restartPolicy: OnFailure
          volumes:
          - name: data-strage
            persistentVolumeClaim:
              claimName: efs-cronjoblog-claim
          - name: config-dir
            emptyDir: {}
          - name: common-config
            configMap:
              name: common-batch-config-map
          - name: block-config
            configMap:
              name: root-common-batch-config-map
          - name: output-user-data-file-mng-batch-config
            configMap:
              name: root-output-user-data-file-mng-batch-container-config-map
```

  - add-region-close-target-batch.yaml
```json
apiVersion: batch/v1beta1
kind: CronJob
metadata:
  name: root-add-region-close-target-batch
  namespace: pxr
spec:
  concurrencyPolicy: Forbid
  successfulJobsHistoryLimit: 1
  failedJobsHistoryLimit: 1
  startingDeadlineSeconds: 3600
  schedule: "0 9 * * * " ★必要に応じてスケジュール修正
  jobTemplate:
    spec:
      template:
        spec:
          containers:
          - name: add-region-close-target-batch
            image: ＜ECRのDNS名＞/pxross1:add-region-close-target-batch-container ★ECRのDNS名を修正
            imagePullPolicy: IfNotPresent
            command: ["node", "/usr/src/app/addRegionCloseTarget.js"]
            volumeMounts:
            - mountPath: /usr/src/app/efs
              name: data-strage
            - mountPath: /usr/src/app/config
              readOnly: true
              name: add-region-close-target-batch-config
          restartPolicy: OnFailure
          volumes:
          - name: data-strage
            persistentVolumeClaim:
              claimName: efs-cronjoblog-claim
          - name: config-dir
            emptyDir: {}
          - name: common-config
            configMap:
              name: common-batch-config-map
          - name: block-config
            configMap:
              name: root-common-batch-config-map
          - name: add-region-close-target-batch-config
            configMap:
              name: root-add-region-close-target-batch-container-config-map
```

  - close-region-batch.yaml
```json
apiVersion: batch/v1beta1
kind: CronJob
metadata:
  name: root-close-region-batch
  namespace: pxr
spec:
  concurrencyPolicy: Forbid
  successfulJobsHistoryLimit: 1
  failedJobsHistoryLimit: 1
  startingDeadlineSeconds: 3600
  schedule: "30 9 * * * " ★必要に応じてスケジュール修正
  jobTemplate:
    spec:
      template:
        spec:
          containers:
          - name: close-region-batch
            image: ＜ECRのDNS名＞/pxross1:close-region-batch-container ★ECRのDNS名を修正
            imagePullPolicy: IfNotPresent
            command: ["node", "/usr/src/app/closeRegion.js"]
            volumeMounts:
            - mountPath: /usr/src/app/efs
              name: data-strage
            - mountPath: /usr/src/app/config
              readOnly: true
              name: close-region-batch-config
          restartPolicy: OnFailure
          volumes:
          - name: data-strage
            persistentVolumeClaim:
              claimName: efs-cronjoblog-claim
          - name: config-dir
            emptyDir: {}
          - name: common-config
            configMap:
              name: common-batch-config-map
          - name: block-config
            configMap:
              name: root-common-batch-config-map
          - name: close-region-batch-config
            configMap:
              name: root-close-region-batch-container-config-map
```
### region000001
　```★の箇所を修正する```
  - region-end-of-use-batch.yaml
```json
apiVersion: batch/v1beta1
kind: CronJob
metadata:
  name: region000001-region-end-of-use-batch
  namespace: pxr
spec:
  concurrencyPolicy: Forbid
  successfulJobsHistoryLimit: 1
  failedJobsHistoryLimit: 1
  startingDeadlineSeconds: 3600
  schedule: "5 * * * *" ★必要に応じてスケジュール修正
  jobTemplate:
    spec:
      template:
        spec:
          containers:
          - name: region-end-of-use-batch
            image: ＜ECRのDNS名＞/pxross1:region-end-of-use-batch-container ★ECRのDNS名を修正
            imagePullPolicy: IfNotPresent
            command: ["node", "/usr/src/app/regionEndOfUse.js"]
            volumeMounts:
            - mountPath: /usr/src/app/efs
              name: data-strage
            - mountPath: /usr/src/app/config
              readOnly: true
              name: region-end-of-use-batch-config
          restartPolicy: OnFailure
          volumes:
          - name: data-strage
            persistentVolumeClaim:
              claimName: efs-cronjoblog-claim
          - name: config-dir
            emptyDir: {}
          - name: common-config
            configMap:
              name: common-batch-config-map
          - name: block-config
            configMap:
              name: root-common-batch-config-map
          - name: region-end-of-use-batch-config
            configMap:
              name: region000001-region-end-of-use-batch-container-config-map
```

  - book-delete-batch.yaml
```json
apiVersion: batch/v1beta1
kind: CronJob
metadata:
  name: region000001-book-delete-batch
  namespace: pxr
spec:
  concurrencyPolicy: Forbid
  successfulJobsHistoryLimit: 1
  failedJobsHistoryLimit: 1
  startingDeadlineSeconds: 3600
  schedule: "0 8 * * *" ★必要に応じてスケジュール修正
  jobTemplate:
    spec:
      template:
        spec:
          containers:
          - name: book-delete-batch
            image: ＜ECRのDNS名＞/pxross1:book-delete-batch-container ★ECRのDNS名を修正
            imagePullPolicy: IfNotPresent
            command: ["node", "/usr/src/app/deleteBook.js"]
            volumeMounts:
            - mountPath: /usr/src/app/efs
              name: data-strage
            - mountPath: /usr/src/app/config
              readOnly: true
              name: book-delete-batch-config
          restartPolicy: OnFailure
          volumes:
          - name: data-strage
            persistentVolumeClaim:
              claimName: efs-cronjoblog-claim
          - name: config-dir
            emptyDir: {}
          - name: common-config
            configMap:
              name: common-batch-config-map
          - name: block-config
            configMap:
              name: root-common-batch-config-map
          - name: book-delete-batch-config
            configMap:
              name: region000001-book-delete-batch-container-config-map
```

  - cooperate-region-batch.yaml
```json
apiVersion: batch/v1beta1
kind: CronJob
metadata:
  name: region000001-cooperate-region-batch
  namespace: pxr
spec:
  concurrencyPolicy: Forbid
  successfulJobsHistoryLimit: 1
  failedJobsHistoryLimit: 1
  startingDeadlineSeconds: 3600
  schedule: "*/5 * * * *" ★必要に応じてスケジュール修正
  jobTemplate:
    spec:
      template:
        spec:
          containers:
          - name: cooperate-region-batch
            image: ＜ECRのDNS名＞/pxross1:cooperate-region-batch-container ★ECRのDNS名を修正
            imagePullPolicy: IfNotPresent
            command: ["node", "/usr/src/app/cooperateRegion.js"]
            volumeMounts:
            - mountPath: /usr/src/app/efs
              name: data-strage
            - mountPath: /usr/src/app/config
              readOnly: true
              name: cooperate-region-batch-config
          restartPolicy: OnFailure
          volumes:
          - name: data-strage
            persistentVolumeClaim:
              claimName: efs-cronjoblog-claim
          - name: config-dir
            emptyDir: {}
          - name: common-config
            configMap:
              name: common-batch-config-map
          - name: block-config
            configMap:
              name: root-common-batch-config-map
          - name: cooperate-region-batch-config
            configMap:
              name: region000001-cooperate-region-batch-container-config-map
```
### Application000001/2/3
　```★の箇所を修正する```
  - create-user-batch.yaml
```json
apiVersion: batch/v1beta1
kind: CronJob
metadata:
  name: application000001-create-user-batch
  namespace: pxr
spec:
  concurrencyPolicy: Forbid
  successfulJobsHistoryLimit: 1
  failedJobsHistoryLimit: 1
  startingDeadlineSeconds: 3600
  schedule: "10-59/15 * * * *" ★必要に応じてスケジュール修正
  jobTemplate:
    spec:
      template:
        spec:
          containers:
          - name: create-user-batch
            image: ＜ECRのDNS名＞/pxross1:create-user-batch-container ★ECRのDNS名を修正
            imagePullPolicy: IfNotPresent
            command: ["node", "/usr/src/app/CreateUser.js"]
            volumeMounts:
            - mountPath: /usr/src/app/efs
              name: data-strage
            - mountPath: /usr/src/app/config
              readOnly: true
              name: create-user-batch-config
          restartPolicy: OnFailure
          volumes:
          - name: data-strage
            persistentVolumeClaim:
              claimName: efs-cronjoblog-claim
          - name: config-dir
            emptyDir: {}
          - name: common-config
            configMap:
              name: common-batch-config-map
          - name: block-config
            configMap:
              name: root-common-batch-config-map
          - name: create-user-batch-config
            configMap:
              name: application000001-create-user-batch-container-config-map
```

  - link-ctoken-batch.yaml
```json
apiVersion: batch/v1beta1
kind: CronJob
metadata:
  name: application000001-link-ctoken-batch
  namespace: pxr
spec:
  concurrencyPolicy: Forbid
  successfulJobsHistoryLimit: 1
  failedJobsHistoryLimit: 1
  startingDeadlineSeconds: 3600
  schedule: "5-59/15 * * * *" ★必要に応じてスケジュール修正
  jobTemplate:
    spec:
      template:
        spec:
          containers:
          - name: link-ctoken-batch
            image: ＜ECRのDNS名＞/pxross1:link-ctoken-batch-container ★ECRのDNS名を修正
            imagePullPolicy: IfNotPresent
            command: ["node", "/usr/src/app/linkCToken.js"]
            volumeMounts:
            - mountPath: /usr/src/app/efs
              name: data-strage
            - mountPath: /usr/src/app/config
              readOnly: true
              name: link-ctoken-batch-config
          restartPolicy: OnFailure
          volumes:
          - name: data-strage
            persistentVolumeClaim:
              claimName: efs-cronjoblog-claim
          - name: config-dir
            emptyDir: {}
          - name: common-config
            configMap:
              name: common-batch-config-map
          - name: block-config
            configMap:
              name: application000001-common-batch-config-map
          - name: link-ctoken-batch-config
            configMap:
              name: application000001-link-ctoken-batch-container-config-map
```

  - book-delete-batch.yaml
```json
apiVersion: batch/v1beta1
kind: CronJob
metadata:
  name: application000001-book-delete-batch
  namespace: pxr
spec:
  concurrencyPolicy: Forbid
  successfulJobsHistoryLimit: 1
  failedJobsHistoryLimit: 1
  startingDeadlineSeconds: 3600
  schedule: "0 8 * * *" ★必要に応じてスケジュール修正
  jobTemplate:
    spec:
      template:
        spec:
          containers:
          - name: book-delete-batch
            image: ＜ECRのDNS名＞/pxross1:book-delete-batch-container ★ECRのDNS名を修正
            imagePullPolicy: IfNotPresent
            command: ["node", "/usr/src/app/deleteBook.js"]
            volumeMounts:
            - mountPath: /usr/src/app/efs
              name: data-strage
            - mountPath: /usr/src/app/config
              readOnly: true
              name: book-delete-batch-config
          restartPolicy: OnFailure
          volumes:
          - name: data-strage
            persistentVolumeClaim:
              claimName: efs-cronjoblog-claim
          - name: config-dir
            emptyDir: {}
          - name: common-config
            configMap:
              name: common-batch-config-map
          - name: block-config
            configMap:
              name: application000001-common-batch-config-map
          - name: book-delete-batch-config
            configMap:
              name: application000001-book-delete-batch-container-config-map
```

  - delete-user-batch.yaml
```json
apiVersion: batch/v1beta1
kind: CronJob
metadata:
  name: application000001-delete-user-batch
  namespace: pxr
spec:
  concurrencyPolicy: Forbid
  successfulJobsHistoryLimit: 1
  failedJobsHistoryLimit: 1
  startingDeadlineSeconds: 3600
  schedule: "45 * * * *" ★必要に応じてスケジュール修正
  jobTemplate:
    spec:
      template:
        spec:
          containers:
          - name: delete-user-batch
            image: ＜ECRのDNS名＞/pxross1:delete-user-batch-container ★ECRのDNS名を修正
            imagePullPolicy: IfNotPresent
            command: ["node", "/usr/src/app/deleteUser.js"]
            volumeMounts:
            - mountPath: /usr/src/app/efs
              name: data-strage
            - mountPath: /usr/src/app/config
              readOnly: true
              name: delete-user-batch-config
          restartPolicy: OnFailure
          volumes:
          - name: data-strage
            persistentVolumeClaim:
              claimName: efs-cronjoblog-claim
          - name: config-dir
            emptyDir: {}
          - name: common-config
            configMap:
              name: common-batch-config-map
          - name: block-config
            configMap:
              name: root-common-batch-config-map
          - name: delete-user-batch-config
            configMap:
              name: application000001-delete-user-batch-container-config-map
```

## configmapの各アクターに対して該当バッチのyaml配置及び設定変更
  - push方法 : 「\Box\External：PxRドキュメント共有用\EX_PxR_NEC-NES-Partner\PxR_業務分析_パートナー共有\04.ドキュメント整備\03.執筆\10_セミナー向け\02_構築事業者向け\work\バッチサンプル\# リリース手順_eval3結合リリース手順追記.md」の「### eval3 結合試験用 リリース　 ★」を参照すること
    - ★配置パスは適切な場所に移動後、修正すること
  - パス : /root/work/build/manifest/eks/configmap
 
### root
　```★の箇所を修正する```
  - region-term-of-use-notification-batch-container.yaml
```json
apiVersion: v2
kind: ConfigMap
metadata:
  name: root-region-term-of-use-notification-batch-container-config-map
  namespace: pxr
data:
  config.json: |
    {
      "catalogService": {
        "getCatalogName": "http://root-service/pxr-block-proxy/pxr-block-proxy/?from_path=%2Fpxr-book-manage&block=$pxr-root-block&path=%2Fcatalog%2Fname",
        "getForNs": "http://root-service/pxr-block-proxy/pxr-block-proxy/?from_path=%2Fpxr-book-manage&block=$pxr-root-block&path=%2Fcatalog%2F%3Fns%3D"
      },
      "notificationService": {
        "register": "http://root-service/pxr-block-proxy/pxr-block-proxy/?from_path=%2Fpxr-book-manage&block=$pxr-root-block&path=%2Fnotification"
      },
      "bookManageService": {
        "getRegionTargetFind": "http://root-service/pxr-block-proxy/pxr-block-proxy/?from_path=%2Fpxr-book-manage&block=$pxr-root-block&path=%2Fbook-manage%2Fterm_of_use%2Fregion%2Ftarget%2Ffind",
        "getRegionTarget": "http://root-service/pxr-block-proxy/pxr-block-proxy/?from_path=%2Fpxr-book-manage&block=$pxr-root-block&path=%2Fbook-manage%2Fterm_of_use%2Fregion%2Ftarget%3Foffset%3D{offset}%26limit%3D10",
        "getRegionNotificationComplete": "http://root-service/pxr-block-proxy/pxr-block-proxy/?from_path=%2Fpxr-book-manage&block=$pxr-root-block&path=%2Fbook-manage%2Fterm_of_use%2Fregion%2Fnotification%2Fcomplete"
      },
      "session": {
        "loginId": "root_batch",
        "auth": {
          "member": {
            "add": true,
            "update": true
          },
          "book": {
            "create": true
          }
        },
        "block": {
          "_value": 1000401, // ★rootのblockコード
          "_ver": 1 // ★rootのblockバージョン
        },
        "actor": {
          "_value": 1000431, // ★rootのactorコード
          "_ver": 1 // ★rootのactorバージョン
        }
      },
      "catalog": {
        "globalSettingNs": "catalog/ext/{extName}/setting/global",
        "pxrRootBlockNs": "catalog/ext/{extName}/block/pxr-root"
      },
      "timezone": "Asia/Tokyo"
    }
```

  - book-end-of-use-batch-container.yaml
```json
apiVersion: v1
kind: ConfigMap
metadata:
  name: root-book-end-of-use-batch-container-config-map
  namespace: pxr
data:
  config.json: |
    {
      "bookManageService": {
        "getPlatformDeletionTarget": "http://root-service/pxr-block-proxy/pxr-block-proxy/?from_path=%2Fpxr-book-manage&block=$pxr-root-block&path=%2Fbook-manage%2Fterm_of_use%2Fplatform%2Fdeletion%2Ftarget%3Foffset%3D{offset}%26limit%3D10",
        "forceDeletion": "http://root-service/pxr-block-proxy/pxr-block-proxy/?from_path=%2Fpxr-book-manage&block=$pxr-root-block&path=%2Fbook-manage%2Fforce-deletion",
        "forceEnable": "http://root-service/pxr-block-proxy/pxr-block-proxy/?from_path=%2Fpxr-book-manage&block=$pxr-root-block&path=%2Fbook-manage%2Fforce-enable",
        "postGetOutputCode": "http://root-service/pxr-block-proxy/pxr-block-proxy/?from_path=%2Fpxr-book-manage&block=$pxr-root-block&path=%2Fbook-manage%2Foutput%3Foffset%3D{offset}%26limit%3D10",
        "postOutputPrepare": "http://root-service/pxr-block-proxy/pxr-block-proxy/?from_path=%2Fpxr-book-manage&block=$pxr-root-block&path=%2Fbook-manage%2Foutput%2Fcondition%2Fprepare"
      },
      "session": {
        "loginId": "root_batch",
        "auth": {
          "member": {
            "add": true,
            "update": true
          },
          "book": {
            "create": true
          }
        },
        "block": {
          "_value": 1000401, // ★rootのblockコード
          "_ver": 1 // ★rootのblockバージョン
        },
        "actor": {
          "_value": 1000431, // ★rootのactorコード
          "_ver": 1 // ★rootのactorバージョン
        }
      },
      "timezone": "Asia/Tokyo"
    }
```

  - add-region-end-of-use-target-batch-container.yaml
```json
apiVersion: v1
kind: ConfigMap
metadata:
  name: root-add-region-end-of-use-target-batch-container-config-map
  namespace: pxr
data:
  config.json: |
    {
      "bookManageService": {
        "getRegionDeletionTarget": "http://root-service/pxr-block-proxy/pxr-block-proxy/?from_path=%2Fpxr-book-manage&block=$pxr-root-block&path=%2Fbook-manage%2Fterm_of_use%2Fregion%2Fdeletion%2Ftarget%3Foffset%3D{offset}%26limit%3D10",
        "postGetOutputCode": "http://root-service/pxr-block-proxy/pxr-block-proxy/?from_path=%2Fpxr-book-manage&block=$pxr-root-block&path=%2Fbook-manage%2Foutput%3Foffset%3D{offset}%26limit%3D10",
        "postOutputPrepare": "http://root-service/pxr-block-proxy/pxr-block-proxy/?from_path=%2Fpxr-book-manage&block=$pxr-root-block&path=%2Fbook-manage%2Foutput%2Fcondition%2Fprepare"
      },
      "catalogService": {
        "getCatalogName": "http://root-service/pxr-block-proxy/pxr-block-proxy/?from_path=%2Fpxr-book-manage&block=$pxr-root-block&path=%2Fcatalog%2Fname",
        "getForNs": "http://root-service/pxr-block-proxy/pxr-block-proxy/?from_path=%2Fpxr-book-manage&block=$pxr-root-block&path=%2Fcatalog%2F%3Fns%3D"
      },
      "session": {
        "loginId": "root_batch",
        "auth": {
          "member": {
            "add": true,
            "update": true
          },
          "book": {
            "create": true
          }
        },
        "block": {
          "_value": 1000401, // ★rootのblockコード
          "_ver": 1 // ★rootのblockバージョン
        },
        "actor": {
          "_value": 1000431, // ★rootのactorコード
          "_ver": 1 // ★rootのactorバージョン
        }
      },
      "catalog": {
        "globalSettingNs": "catalog/ext/{extName}/setting/global",
        "pxrRootBlockNs": "catalog/ext/{extName}/block/pxr-root"
      },
      "timezone": "Asia/Tokyo"
    }
```

  - book-delete-batch-container.yaml
```json
apiVersion: v1
kind: ConfigMap
metadata:
  name: root-book-delete-batch-container-config-map
  namespace: pxr
data:
  config.json: |
    {
      "catalogService": {
        "getForNs": "http://root-service/pxr-block-proxy/pxr-block-proxy/?from_path=%2Fpxr-book-manage&block=$pxr-root-block&path=%2Fcatalog%2F%3Fns%3D",
        "getForCode": "http://root-service/pxr-block-proxy/pxr-block-proxy/?from_path=%2Fpxr-book-manage&block=$pxr-root-block&path=%2Fcatalog%2F"
      },
      "bookManageService": {
        "getDeleteTarget": "http://root-service/pxr-block-proxy/pxr-block-proxy/?from_path=%2Fpxr-book-manage&block=$pxr-root-block&path=%2Fbook-manage%2Fdelete%2Ftarget%3Foffset%3D{offset}%26limit%3D10",
        "postSearchBook": "http://root-service/pxr-block-proxy/pxr-block-proxy/?from_path=%2Fpxr-book-manage&block=$pxr-root-block&path=%2Fbook-manage%2Fsearch",
        "postCooperateRelease": "http://root-service/pxr-block-proxy/pxr-block-proxy/?from_path=%2Fpxr-book-manage&block=$pxr-root-block&path=%2Fbook-manage%2Fcooperate%2Frelease",
        "deleteForce": "http://root-service/pxr-block-proxy/pxr-block-proxy/?from_path=%2Fpxr-book-manage&block=$pxr-root-block&path=%2Fbook-manage%2Fforce%2F{pxrId}%3FphysicalDelete%3D{physicalDelete}"
      },
      "identityVerificateService": {
        "postCodeVerified": "http://root-service/pxr-block-proxy/pxr-block-proxy/?from_path=%2Fpxr-book-manage&block=$pxr-root-block&path=%2Fidentity-verificate%2Fcode%2Fverified"
      },
      "bookOperateService": {
          "postUserDelete": ""
      },
      "catalog": {
        "platformTermsOfUseNs": "catalog/ext/nec-healthcare-consortium/terms-of-use/platform" // ★該当ネームスペースのパスを環境に合わせて修正 
      },
      "session": {
        "loginId": "root_batch",
        "auth": {
          "member": {
            "delete": true
          },
          "book": {
              "create": true
          }
        },
        "block": {
          "_value": 1000401, // ★ rootのblockコード
          "_ver": 1 // ★ rootのblockバージョン
        },
        "actor": {
          "_value": 1000431, // ★ rootのactorコード
          "_ver": 1 // ★rootのactorバージョン
        }
      },
      "rootSession": {
          "loginId": "root_batch",
          "block": {
            "_value": 1000401, // ★ rootのblockコード
            "_ver": 1 // ★ rootのblockバージョン
          },
          "actor": {
            "_value": 1000431, // ★ rootのactorコード
            "_ver": 1 // ★rootのactorバージョン
          }
      },
      "releaseSession": {
          "loginId": "release_id_batch",
          "type": 3,
          "block": {
            "_value": 1000401, // ★ rootのblockコード
            "_ver": 1 // ★ rootのblockバージョン
          },
          "actor": {
            "_value": 0,
            "_ver": 0
          }
      },
      "timezone": "Asia/Tokyo"
    }
```
  - output-user-data-file-mng-batch-container.yaml
```json
apiVersion: v1
kind: ConfigMap
metadata:
  name: root-output-user-data-file-mng-batch-container-config-map
  namespace: pxr
data:
  config.json: |
    {
      "catalogService": {
        "getCatalogName": "http://root-service/pxr-block-proxy/pxr-block-proxy/?from_path=%2Fpxr-book-manage&block=$pxr-root-block&path=%2Fcatalog%2Fname",
        "getForNs": "http://root-service/pxr-block-proxy/pxr-block-proxy/?from_path=%2Fpxr-book-manage&block=$pxr-root-block&path=%2Fcatalog%2F%3Fns%3D",
        "getForCode": "http://root-service/pxr-block-proxy/pxr-block-proxy/?from_path=%2Fpxr-book-manage&block=$pxr-root-block&path=%2Fcatalog%2F{code}%2F{version}",
        "getCatalog": "http://root-service/pxr-block-proxy/pxr-block-proxy/?from_path=%2Fpxr-book-manage&block=$pxr-root-block&path=%2Fcatalog"
      },
      "bookManageService": {
        "getOutputCondition": "http://root-service/pxr-block-proxy/pxr-block-proxy/?from_path=%2Fpxr-book-manage&block=$pxr-root-block&path=%2Fbook-manage%2Foutput%2Fcondition%3Foffset%3D{offset}%26limit%3D10%26approved%3D1",
        "postGetMcdOutputCodeDataFile": "http://root-service/pxr-block-proxy/pxr-block-proxy/?from_path=%2Fpxr-book-manage&block=$pxr-root-block&path=%2Fbook-manage%2Foutput%2Fcondition%2Fdata_mng%2Fsearch",
        "postCreateMcdOutputCodeDataFile": "http://root-service/pxr-block-proxy/pxr-block-proxy/?from_path=%2Fpxr-book-manage&block=$pxr-root-block&path=%2Fbook-manage%2Foutput%2Fcondition%2Fdata_mng%2F",
        "postGetMcdOutput": "http://root-service/pxr-block-proxy/pxr-block-proxy/?from_path=%2Fpxr-book-manage&block=$pxr-root-block&path=%2Fbook-manage%2Foutput%3Foffset%3D0%26limit%3D10",
        "postReserveDeletion": "http://root-service/pxr-block-proxy/pxr-block-proxy/?from_path=%2Fpxr-book-manage&block=$pxr-root-block&path=%2Fbook-manage%2Freserve_deletion%2F{id}"
      },
      "session": {
        "loginId": "root_batch",
        "block": {
          "_value": 1000401, // ★rootのblockコード
          "_ver": 1 // ★rootのblockバージョン
        },
        "actor": {
          "_value": 1000431, // ★rootのactorコード
          "_ver": 1 // ★rootのactorバージョン
        }
      },
      "catalog": {
        "globalSettingNs": "catalog/ext/{extName}/setting/global"
      },
      "timezone": "Asia/Tokyo"
    }
```

  - add-region-close-target-batch.yaml
```json
apiVersion: v1
kind: ConfigMap
metadata:
  name: root-add-region-close-target-batch-container-config-map
  namespace: pxr
data:
  config.json: |
    {
      "catalogService": {
          "getCatalogName": "http://root-service/pxr-block-proxy/pxr-block-proxy/?from_path=%2Fpxr-book-manage&block=$pxr-root-block&path=%2Fcatalog%2Fname",
          "getForNs": "http://root-service/pxr-block-proxy/pxr-block-proxy/?from_path=%2Fpxr-book-manage&block=$pxr-root-block&path=%2Fcatalog%2F%3Fns%3D"
      },
      "bookManageService": {
          "getRegionCloseTarget": "http://root-service/pxr-block-proxy/pxr-block-proxy/?from_path=%2Fpxr-book-manage&block=$pxr-root-block&path=%2Fbook-manage%2Fregion%2Fclose%2Ftarget%3Foffset%3D{offset}%26limit%3D10",
          "postOutput": "http://root-service/pxr-block-proxy/pxr-block-proxy/?from_path=%2Fpxr-book-manage&block=$pxr-root-block&path=%2Fbook-manage%2Foutput%3Foffset%3D0%26limit%3D10",
          "postOutputConditionPrepare": "http://root-service/pxr-block-proxy/pxr-block-proxy/?from_path=%2Fpxr-book-manage&block=$pxr-root-block&path=%2Fbook-manage%2Foutput%2Fcondition%2Fprepare"
      },
      "session": {
          "loginId": "root_batch",
          "auth": {
            "member": {
              "add": true,
              "update": true
            },
            "book": {
              "create": true
            }
          },
          "block": {
            "_value": 1000401, // ★rootのblockコード
            "_ver": 1 // ★rootのblockバージョン
          },
          "actor": {
            "_value": 1000431, // ★rootのactorコード
            "_ver": 1 // ★rootのactorバージョン
          }
      },
      "catalog": {
          "globalSettingNs": "catalog/ext/{extName}/setting/global"
      },
      "timezone": "Asia/Tokyo"
    }
```

  - close-region-batch.yaml
```json
apiVersion: v1
kind: ConfigMap
metadata:
  name: root-close-region-batch-container-config-map
  namespace: pxr
data:
  config.json: |
    {
      "catalogService": {
          "getCatalogName": "http://root-service/pxr-block-proxy/pxr-block-proxy/?from_path=%2Fpxr-book-manage&block=$pxr-root-block&path=%2Fcatalog%2Fname",
          "getCatalog": "http://root-service/pxr-block-proxy/pxr-block-proxy/?from_path=%2Fpxr-book-manage&block=$pxr-root-block&path=%2Fcatalog%2F{code}",
          "getForNs": "http://root-service/pxr-block-proxy/pxr-block-proxy/?from_path=%2Fpxr-book-manage&block=$pxr-root-block&path=%2Fcatalog%2F%3Fns%3D",
          "getByCode": "http://root-service/pxr-block-proxy/pxr-block-proxy/?from_path=%2Fpxr-book-manage&block=$pxr-root-block&path=%2Fcatalog%2F{code}",
          "putCatalogExtCode": "http://root-service/pxr-block-proxy/pxr-block-proxy/?from_path=%2Fpxr-book-manage&block=$pxr-root-block&path=%2Fcatalog%2Fext%2F{code}"
      },
      "bookOperateService": {
          "postUserDelete": "http://root-service/pxr-block-proxy/pxr-block-proxy/?from_path=%2Fpxr-book-manage&block=$pxr-root-block&path=%2Fbook-operate%2Fuser%2Fdelete%3FphysicalDelete%3D{physicalDelete}"
      },
      "bookManageService": {
          "getRegionClose": "http://root-service/pxr-block-proxy/pxr-block-proxy/?from_path=%2Fpxr-book-manage&block=$pxr-root-block&path=%2Fbook-manage%2Fregion%2Fclose%3Foffset%3D{offset}%26limit%3D10",
          "putRegionClose": "http://root-service/pxr-block-proxy/pxr-block-proxy/?from_path=%2Fpxr-book-manage&block=$pxr-root-block&path=%2Fbook-manage%2Fregion%2Fclose",
          "postOutput": "http://root-service/pxr-block-proxy/pxr-block-proxy/?from_path=%2Fpxr-book-manage&block=$pxr-root-block&path=%2Fbook-manage%2Foutput%3Foffset%3D{offset}%26limit%3D10",
          "putOutputId": "http://root-service/pxr-block-proxy/pxr-block-proxy/?from_path=%2Fpxr-book-manage&block=$pxr-root-block&path=%2Fbook-manage%2Foutput%2F{id}",
          "postMcbSearch": "http://root-service/pxr-block-proxy/pxr-block-proxy/?from_path=%2Fpxr-book-manage&block=$pxr-root-block&path=%2Fbook-manage%2Fsearch",
          "postCooperateRelease": "http://root-service/pxr-block-proxy/pxr-block-proxy/?from_path=%2Fpxr-book-manage&block=$pxr-root-block&path=%2Fbook-manage%2Fcooperate%2Frelease",
          "getRegionCloseTarget": "http://root-service/pxr-block-proxy/pxr-block-proxy/?from_path=%2Fpxr-book-manage&block=$pxr-root-block&path=%2Fbook-manage%2Fregion%2Fclose%2Ftarget%3Foffset%3D{offset}%26limit%3D10"
      },
      "identityVerificateService": {
          "postCodeVerified": "http://root-service/pxr-block-proxy/pxr-block-proxy/?from_path=%2Fpxr-book-manage&block=$pxr-root-block&path=%2Fidentity-verificate%2Fcode%2Fverified"
      },
      "session": {
          "loginId": "root_batch",
          "auth": {
              "book": {
                  "create": true
              },
              "catalog": {
                  "update": true
              }
          },
          "block": {
            "_value": 1000401, // ★rootのblockコード
            "_ver": 1 // ★rootのblockバージョン
          },
          "actor": {
              "_value": null,
              "_ver": 1 // ★rootのacotrバージョン
          }
      },
      "releaseSession": {
          "operatorId": 999999999999,
          "loginId": "close-region-batch",
          "type": 3,
          "auth": {
              "member": {
                  "delete": true
              },
              "catalog": {
                  "create": true,
                  "update": true
              }
          },
          "block": {
            "_value": 1000401, // ★rootのblockコード
            "_ver": 1 // ★rootのblockバージョン
          },
          "actor": {
              "_value": 0,
              "_ver": 0
          }
      },
      "closeRegionSession": {
          "loginId": "close-region-batch",
          "type": 3,
          "auth": {
              "book": {
                  "create": true
              },
              "catalog": {
                  "create": true,
                  "update": true
              }
          },
          "block": {
            "_value": 1000401, // ★rootのblockコード
            "_ver": 1 // ★rootのblockバージョン
          },
          "actor": {
              "_value": 0,
              "_ver": 0
          }
      },
      "catalog": {
          "globalSettingNs": "catalog/ext/{extName}/setting/global"
      },
      "timezone": "Asia/Tokyo"
    }
```

### region000001
  - region-end-of-use-batch-container.yaml
```json
apiVersion: v1
kind: ConfigMap
metadata:
  name: region000001-region-end-of-use-batch-container-config-map
  namespace: pxr
data:
  config.json: |
    {
      "catalogService": {
        "getCatalogName": "http://region000001-service/pxr-block-proxy/pxr-block-proxy/?from_path=%2Fpxr-book-manage&block=$pxr-root-block&path=%2Fcatalog%2Fname",
        "getCatalog": "http://region000001-service/pxr-block-proxy/pxr-block-proxy/?from_path=%2Fbook-operate&block=$pxr-root-block&path=%2Fcatalog%2F{code}",
        "getForNs": "http://region000001-service/pxr-block-proxy/pxr-block-proxy/?from_path=%2Fpxr-book-manage&block=$pxr-root-block&path=%2Fcatalog%2F%3Fns%3D"
      },
      "bookOperateService": {
          "postUserDelete": "http://region000001-service/pxr-block-proxy/pxr-block-proxy/?from_path=%2Fbook-operate&block=1000402&path=%2Fbook-operate%2Fuser%2Fdelete%3FphysicalDelete%3D{physicalDelete}"  // ★パスの「block=1000402」はregion000001のblockコード
      },
      "bookManageService": {
        "postGetMcdOutput": "http://region000001-service/pxr-block-proxy/pxr-block-proxy/?from_path=%2Fpxr-book-manage&block=$pxr-root-block&path=%2Fbook-manage%2Foutput%3Foffset%3D{offset}%26limit%3D10",
        "putUpdateMcdOutput": "http://region000001-service/pxr-block-proxy/pxr-block-proxy/?from_path=%2Fpxr-book-manage&block=$pxr-root-block&path=%2Fbook-manage%2Foutput%2F{id}",
        "postMcbSearch": "http://region000001-service/pxr-block-proxy/pxr-block-proxy/?from_path=%2Fpxr-book-manage&block=$pxr-root-block&path=%2Fbook-manage%2Fsearch",
        "postGetMcdDataFile": "http://region000001-service/pxr-block-proxy/pxr-block-proxy/?from_path=%2Fpxr-book-manage&block=$pxr-root-block&path=%2Fbook-manage%2Foutput%2Fcondition%2Fdata_mng%2Fsearch%3Foffset%3D0%26limit%3D0%26approved%3D1",
        "postCooperateReleaseId": "http://region000001-service/pxr-block-proxy/pxr-block-proxy/?from_path=%2Fpxr-book-manage&block=$pxr-root-block&path=%2Fbook-manage%2Fcooperate%2Frelease"
      },
      "identityVerificateService": {
        "postCodeVerified": "http://region000001-service/pxr-block-proxy/pxr-block-proxy/?from_path=%2Fpxr-book-manage&block=$pxr-root-block&path=%2Fidentity-verificate%2Fcode%2Fverified"
      },
      "session": {
        "loginId": "region000001_batch",
        "auth": {
            "member": {
                "delete": true
            },
            "book": {
                "create": true
            }
        },
        "block": {
          "_value": 1000402, // ★region000001のblockコード
          "_ver": 1 // ★region000001のblockバージョン
        },
        "actor": {
          "_value": 1000783, // ★region000001のactorコード
          "_ver": 1 // ★region000001のactorバージョン
        }
      },
      "catalog": {
        "globalSettingNs": "catalog/ext/{extName}/setting/global",
        "regionTouNs": "catalog/ext/{extName}/terms-of-use/region/actor_{actor}"
      },
      "timezone": "Asia/Tokyo"
    }
```

  - book-delete-batch-container.yaml
```json
apiVersion: v1
kind: ConfigMap
metadata:
  name: region000001-book-delete-batch-container-config-map
  namespace: pxr
data:
  config.json: |
    {
      "catalogService": {
        "getForNs": "http://region000001-service/pxr-block-proxy/pxr-block-proxy/?from_path=%2Fbook-operate&block=$pxr-root-block&path=%2Fcatalog%3Fns%3D",
        "getForCode": "http://region000001-service/pxr-block-proxy/pxr-block-proxy/?from_path=%2Fbook-operate&block=$pxr-root-block&path=%2Fcatalog%2F"
      },
      "bookManageService": {
        "getDeleteTarget": "http://region000001-service/pxr-block-proxy/pxr-block-proxy/?from_path=%2Fbook-operate&block=$pxr-root-block&path=%2Fbook-manage%2Fdelete%2Ftarget%3Foffset%3D{offset}%26limit%3D10",
        "postSearchBook": "http://region000001-service/pxr-block-proxy/pxr-block-proxy/?from_path=%2Fbook-operate&block=$pxr-root-block&path=%2Fbook-manage%2Fsearch",
        "deleteForce": "http://region000001-service/pxr-block-proxy/pxr-block-proxy/?from_path=%2Fbook-operate&block=$pxr-root-block&path=%2Fbook-manage%2Fforce%2F{pxrId}%3FphysicalDelete%3D{physicalDelete}"
      },
      "identityVerificateService": {
        "postCodeVerified": "http://region000001-service/pxr-block-proxy/pxr-block-proxy/?from_path=%2Fbook-operate&block=$pxr-root-block&path=%2Fidentity-verificate%2Fcode%2Fverified"
      },
      "bookOperateService": {
          "postUserDelete": "http://region000001-service/pxr-block-proxy/pxr-block-proxy/?from_path=%2Fbook-operate&block=1000402&path=%2Fbook-operate%2Fuser%2Fdelete%3FphysicalDelete%3D{physicalDelete}" // ★パスの「block=1000402」はregion000001のblockコード
      },
      "catalog": {
        "platformTermsOfUseNs": "catalog/ext/nec-healthcare-consortium/terms-of-use/platform"  // ★該当ネームスペースのパスを環境に合わせて修正
      },
      "session": {
          "loginId": "region000001_batch",
          "auth": {
              "member": {
                  "delete": true
              },
              "book": {
                  "create": true
              }
          },
          "block": {
            "_value": 1000402, // ★region000001のblockコード
            "_ver": 1 // ★region000001のblockバージョン
          },
          "actor": {
            "_value": 1000783, // ★region000001のactorコード
            "_ver": 1 // ★region000001のactorバージョン
          }
      },
      "rootSession": {
          "loginId": "root_batch",
          "block": {
              "_value": 1000401,  // ★rootのblockコード
              "_ver": 1  // ★rootのblockバージョン
          },
          "actor": {
              "_value": 1000431,  // ★rootのactorコード
              "_ver": 1  // ★rootのactorバージョン
          }
      },
      "timezone": "Asia/Tokyo"
    }
```

  - cooperate-region-batch-container.yaml
```json
apiVersion: v1
kind: ConfigMap
metadata:
  name: region000001-cooperate-region-batch-container-config-map
  namespace: pxr
data:
  config.json: |
    {
      "catalogService": {
        "getCatalogName": "/catalog/name",
        "getForNs": "/catalog/?ns="
      },
      "pxrBlockProxyService": {
        "postRoot": "http://region000001-service/pxr-block-proxy/pxr-block-proxy/?block=$pxr-root-block&path="
      },
      "notificationService": {
        "getNotification": "http://region000001-service/pxr-block-proxy/pxr-block-proxy/?from_path=%2Fbook-operate&block=1000402&path=%2Fnotification%2F%3Fis_send%3Dfalse%26is_unread%3Dtrue%26is_approval%3Dfalse%26type%3D0%26num%3D0", // ★パスの「block=1000402」はregion000001のblockコード
        "putNotification": "http://region000001-service/pxr-block-proxy/pxr-block-proxy/?from_path=%2Fbook-operate&block=1000402&path=%2Fnotification%2F" // ★パスの「block=1000402」はregion000001のblockコード
      },
      "bookOperateService": {
        "postUser": "http://region000001-service/pxr-block-proxy/pxr-block-proxy/?from_path=%2Fbook-operate&block=1000402&path=%2Fbook-operate%2Fuser" // ★パスの「block=1000402」はregion000001のblockコード
      },
      "identityVerificateService": {
        "postUserSettings": "/identity-verificate/user/settings",
        "putVerifyOthers": "/identity-verificate/verify/others/"
      },
      "session": {
        "loginId": "region000001_batch",
        "auth": {
          "member": {
            "add": true
          },
          "app-wf-user": {
              "create": true
          }
        },
        "block": {
          "_value": 1000402, // ★region000001のblockコード
          "_ver": 1 // ★region000001のblockバージョン
        },
        "actor": {
          "_value": 1000783, // ★region000001のactorコード
          "_ver": 1 // ★region000001のactorバージョン
        }
      },
      "catalog": {
        "globalSettingNs": "catalog/ext/{extName}/setting/global"
      },
      "timezone": "Asia/Tokyo"
    }
```

### Application000001/2/3
  - create-user-batch-container.yaml
```json
apiVersion: v1
kind: ConfigMap
metadata:
  name: application000001-create-user-batch-container-config-map
  namespace: pxr
data:
  config.json: |
    {
      "bookOperateService": {
        "postUserBatch": "http://application000001-service/pxr-block-proxy/pxr-block-proxy/?from_path=%2Fbook-operate&block=1000407&path=%2Fbook-operate%2Fuser%2Fbatch" // ★パスの「block=1000407」はapplication000001のblockコード
      },
      "session": {
        "loginId": "application000001_batch",
        "auth": {
          "member": {
            "add": true,
            "update": true,
            "delete": true
          },
          "app-wf-user": {
            "create": true
          }
        },
        "block": {
          "_value": 1000407, // ★application000001のblockコード
          "_ver": 1 // ★application000001のblockバージョン
        },
        "actor": {
          "_value": 1000789, // ★application000001のactorコード
          "_ver": 1 // ★application000001のactorバージョン
        }
      },
      "timezone": "Asia/Tokyo"
    }
```

  - link-ctoken-batch-container.yaml
```json
apiVersion: v1
kind: ConfigMap
metadata:
  name: application000001-link-ctoken-batch-container-config-map
  namespace: pxr
data:
  config.json: |
    {
      "localCTokenService": {
        "postLedger": "http://application000001-service/pxr-block-proxy/pxr-block-proxy/?from_path=%2Flocal-ctoken&block=1000407&path=%2Flocal-ctoken%2Fledger", // ★パスの「block=1000407」はapplication000001のblockコード
        "getLedgerCount": "http://application000001-service/pxr-block-proxy/pxr-block-proxy/?from_path=%2Flocal-ctoken&block=1000407&path=%2Flocal-ctoken%2Fledger%2Fcount" // ★パスの「block=1000407」はapplication000001のblockコード
      },
      "session": {
        "loginId": "application000001_batch",
        "block": {
          "_value": 1000407, // ★application000001のblockコード
          "_ver": 1 // ★application000001のblockバージョン
        },
        "actor": {
          "_value": 1000789, // ★application000001のactorコード
          "_ver": 1 // ★application000001のactorバージョン
        }
      },
      "timezone": "Asia/Tokyo"
    }
```
  - book-delete-batch-container.yaml
```json
apiVersion: v1
kind: ConfigMap
metadata:
  name: application000001-book-delete-batch-container-config-map
  namespace: pxr
data:
  config.json: |
    {
      "catalogService": {
        "getForNs": "http://application000001-service/pxr-block-proxy/pxr-block-proxy/?from_path=%2Fbook-operate&block=$pxr-root-block&path=%2Fcatalog%3Fns%3D",
        "getForCode": "http://application000001-service/pxr-block-proxy/pxr-block-proxy/?from_path=%2Fbook-operate&block=$pxr-root-block&path=%2Fcatalog%2F"
      },
      "bookManageService": {
        "getDeleteTarget": "http://application000001-service/pxr-block-proxy/pxr-block-proxy/?from_path=%2Fbook-operate&block=$pxr-root-block&path=%2Fbook-manage%2Fdelete%2Ftarget%3Foffset%3D{offset}%26limit%3D10",
        "postSearchBook": "http://application000001-service/pxr-block-proxy/pxr-block-proxy/?from_path=%2Fbook-operate&block=$pxr-root-block&path=%2Fbook-manage%2Fsearch",
        "deleteForce": "http://application000001-service/pxr-block-proxy/pxr-block-proxy/?from_path=%2Fbook-operate&block=$pxr-root-block&path=%2Fbook-manage%2Fforce%2F{pxrId}%3FphysicalDelete%3D{physicalDelete}"
      },
      "identityVerificateService": {
        "postCodeVerified": "http://application000001-service/pxr-block-proxy/pxr-block-proxy/?from_path=%2Fbook-operate&block=$pxr-root-block&path=%2Fidentity-verificate%2Fcode%2Fverified"
      },
      "bookOperateService": {
          "postUserDelete": "http://application000001-service/pxr-block-proxy/pxr-block-proxy/?from_path=%2Fbook-operate&block=1000407&path=%2Fbook-operate%2Fuser%2Fdelete%3FphysicalDelete%3D{physicalDelete}" // ★パスの「block=1000407」はapplication000001のblockコード
      },
      "catalog": {
        "platformTermsOfUseNs": "catalog/ext/nec-healthcare-consortium/terms-of-use/platform" // ★該当ネームスペースのパスを環境に合わせて修正
      },
      "session": {
          "loginId": "application000001_batch",
          "auth": {
              "member": {
                  "delete": true
              },
              "book": {
                  "create": true
              }
          },
          "block": {
            "_value": 1000407, // ★application000001のblockコード
            "_ver": 1 // ★application000001のblockバージョン
          },
          "actor": {
            "_value": 1000789, // ★application000001のactorコード
            "_ver": 1 // ★applictaion000001のactorバージョン
          }
      },
      "rootSession": {
          "loginId": "root_batch",
          "block": {
              "_value": 1000401, // ★rootのblockコード
              "_ver": 1 // ★rootのblockバージョン
          },
          "actor": {
              "_value": 1000431, // ★rootのactorコード
              "_ver": 1 // ★rootのactorバージョン
          }
      },
      "timezone": "Asia/Tokyo"
    }
```

  - delete-user-batch-container.yaml
```json
apiVersion: v1
kind: ConfigMap
metadata:
  name: application000001-delete-user-batch-container-config-map
  namespace: pxr
data:
  config.json: |
    {
      "catalogService": {
        "getCatalogName": "http://application000001-service/pxr-block-proxy/pxr-block-proxy/?from_path=%2Fbook-operate&block=$pxr-root-block&path=%2Fcatalog%2Fname",
        "getCatalog": "http://application000001-service/pxr-block-proxy/pxr-block-proxy/?from_path=%2Fbook-operate&block=$pxr-root-block&path=%2Fcatalog%2F{code}",
        "getForNs": "http://application000001-service/pxr-block-proxy/pxr-block-proxy/?from_path=%2Fbook-operate&block=$pxr-root-block&path=%2Fcatalog%2F%3Fns%3D"
      },
      "bookManageService": {
        "postGetMcdOutputCodeDataFile": "http://application000001-service/pxr-block-proxy/pxr-block-proxy/?from_path=%2Fbook-operate&block=$pxr-root-block&path=%2Fbook-manage%2Foutput%2Fcondition%2Fdata_mng%2Fsearch%3Foffset%3D{offset}%26limit%3D10%26approved%3D{approved}",
        "postUpdateMcdOutputCodeDataFile": "http://application000001-service/pxr-block-proxy/pxr-block-proxy/?from_path=%2Fbook-operate&block=$pxr-root-block&path=%2Fbook-manage%2Foutput%2Fcondition%2Fdata_mng%2F{id}",
        "postMcbSearch": "http://application000001-service/pxr-block-proxy/pxr-block-proxy/?from_path=%2Fbook-operate&block=$pxr-root-block&path=%2Fbook-manage%2Fsearch%3FdisableFlg%3D1"
      },
      "bookOperateService": {
        "postUserList": "http://application000001-service/pxr-block-proxy/pxr-block-proxy/?from_path=%2Fbook-operate&block=1000407&path=%2Fbook-operate%2Fuser%2Flist", // ★パスの「block=1000407」はapplication000001のblockコード
        "deleteUserData": "http://application000001-service/pxr-block-proxy/pxr-block-proxy/?from_path=%2Fbook-operate&block=1000407&path=%2Fbook-operate%2Fstore%2F{userId}%3FphysicalDelete%3D{physicalDelete}" // ★パスの「block=1000407」はapplication000001のblockコード
      },
      "session": {
        "loginId": "application000001_batch",
        "auth": {
            "book": {
                "create": true
            }
        },
        "block": {
          "_value": 1000407, // ★application000001のblockコード
          "_ver": 1 // ★application000001のblockバージョン
        },
        "actor": {
          "_value": 1000789, // ★application000001のactorコード
          "_ver": 1 // ★application000001のactorバージョン
        }
      },
      "catalog": {
        "globalSettingNs": "catalog/ext/{extName}/setting/global",
        "pxrRootBlockNs": "catalog/ext/{extName}/block/pxr-root",
        "platformTouNs": "catalog/ext/{extName}/terms-of-use/platform",
        "regionTouNs": "catalog/ext/{extName}/terms-of-use/region/actor_{actor}"
      },
      "timezone": "Asia/Tokyo"
    }
```

## manifestをdeploymentする
  - deployment方法 : 
    - manifest設定（各ファイルにて実施） : 
    - 参考資料「\Box\External：PxRドキュメント共有用\EX_PxR_NEC-NES-Partner\PxR_業務分析_パートナー共有\04.ドキュメント整備\03.執筆\10_セミナー向け\02_構築事業者向け\work\バッチサンプル\# リリース手順_eval3結合リリース手順追記.md」の「### eval3 結合試験用 リリース　 ★」を参照すること
    - ★配置パスは適切な場所に移動後、修正すること
  - cronjob動作確認 : 
    - kubectl get CronJob -n pxr 又は kubectl get cj -n {namespace}

## 動作確認
  - バッチスケジュール情報 : \Box\EX_PxR_NEC-NES-Partner\NES_品質強化\②バグ出し活動\施策No.13_バッチスクリプト\PxR_バッチスケジュール定義書_v1.0.xlsx
  - manifestの修正でスケジュールを変更することは可能
  - 基本上記のファイルの起動間隔で実施
  - スケジュールによる実施完了後下記のコマンドでバッチログ確認
    - kubectl logs -n pxr {pod_name}
  - 補足
    - バッチのPODは起動から処理終了までの間しか存在しないので、そのタイミングでログ収集する
    - 一覧は以下のコマンドで取得
      - kubectl get pod -n pxross1 | grep batch